// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "main.h"
#include "compdraw.h"

using namespace os;

CompDraw :: CompDraw( const Rect& cFrame, uint32 nResizeMask ) : View( cFrame, "comp_draw", nResizeMask, WID_WILL_DRAW )
{
	m_SnapMode = true;
	m_SnapX = 10;
	m_SnapY = 10;
	m_SimulationTime = 0;
}

CompDraw :: ~CompDraw()
{
}

void CompDraw :: Paint( const Rect& cUpdate )
{
	// Draw background colour
	SetEraseColor( 255, 255, 255 );
	EraseRect( cUpdate );

	// Draw selection box for all components
	for( int i = 0 ; i < m_ListOfComponents.size() ; i++ )
	{
		Rect cFrame( m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ), m_ListOfComponents[ i ]->GetPosition() + m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) );
		if( cFrame.DoIntersect( cUpdate ) )
		{
			m_ListOfComponents[ i ]->DrawSelectionBox( this );
		}
    }

	// For each component inside the cUpdate frame
	for( int i = 0 ; i < m_ListOfComponents.size() ; i++ )
	{
		Rect cFrame( m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ), m_ListOfComponents[ i ]->GetPosition() + m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) );
		if( cUpdate.DoIntersect( cFrame ) )
			m_ListOfComponents[ i ]->DrawComponent( this );
    }


	// Draw each line segment
	for( int i = 0 ; i < m_ListOfLineSegments.size() ; i++ )
	{
		Rect cFrame(  );
//		if( cFrame.DoIntersect( cUpdate ) )
		{
			m_ListOfLineSegments[ i ]->DrawSegment( this );
		}
    }

}

void CompDraw :: AddComponent( Component* comp )
{
	m_ListOfComponents.push_back( comp );
}

// Return what component is under the cursor
int CompDraw :: HitTest( const Point& cPosition, Point bound )
{
	// Assign position
	for( uint i = 0; i < m_ListOfComponents.size(); i++ )
	{		                        
		// Create frame
		Rect cFrame( m_ListOfComponents[ i ]->GetPosition() - bound, m_ListOfComponents[ i ]->GetPosition() + m_ListOfComponents[ i ]->GetSize() + bound );
                        
		if( cFrame.DoIntersect( cPosition ) )
			return( i );
	}
	return -1;
}
        
// Deselect all components
void CompDraw :: DeselectAll()
{
	for( uint i = 0; i < m_ListOfComponents.size(); i++ )
	{
		Rect cFrame( m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ), 
						 m_ListOfComponents[ i ]->GetPosition() + m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) );

		if( m_ListOfComponents[ i ]->IsSelected() && GetBounds().DoIntersect( cFrame ) )
			Invalidate( cFrame );
                        
		m_ListOfComponents[ i ]->SetSelected( false );
	}
}
        
// Select one component
void CompDraw :: Select( uint nComp, bool bSelected )
{
	if( m_ListOfComponents[ nComp ]->IsSelected() == bSelected )
		return;

	m_ListOfComponents[ nComp ]->SetSelected( bSelected );
	Invalidate( Rect( m_ListOfComponents[ nComp ]->GetPosition() - Point( 3, 3 ), 
			          m_ListOfComponents[ nComp ]->GetPosition() + m_ListOfComponents[ nComp ]->GetSize() + Point( 3, 3 ) ) );
}
        
// Select the icons in this region
void CompDraw :: Select( Point cStart, Point cEnd, bool bKeepSelection )
{
	if( !bKeepSelection )
		DeselectAll();
                        
	// Swap positions if necessary
	if( cStart.x > cEnd.x )
	{
		float vTemp = cStart.x;
		cStart.x = cEnd.x;
		cEnd.x = vTemp;
	}       
	if( cStart.y > cEnd.y ) 
	{
		float vTemp = cStart.y;
		cStart.y = cEnd.y;
		cEnd.y = vTemp;
	}                               
                                
	// Create frame
	Rect cSelectFrame( cStart, cEnd );
    
	// Check positions
	for( uint i = 0; i < m_ListOfComponents.size(); i++ )
	{
		Point cCompPos = Point( m_ListOfComponents[ i ]->GetPosition().x + m_ListOfComponents[ i ]->GetSize().x / 2,
								m_ListOfComponents[ i ]->GetPosition().y );
                        
		// Create frame
		Rect cFrame( cCompPos, cCompPos + m_ListOfComponents[ i ]->GetSize() );
                        
		// Select/deselect
		if( cFrame.DoIntersect( cSelectFrame ) )
		{
			m_ListOfComponents[ i ]->SetSelected( !m_ListOfComponents[ i ]->IsSelected() );
		}
	}
}

void CompDraw :: MouseDown( const Point& cPosition, uint32 nButtons )
{
	if( SLogicApp::GetAppWindow()->GetCurrentTool() != NULL )
		SLogicApp::GetAppWindow()->GetCurrentTool()->MouseDown( this, cPosition, nButtons );
}

void CompDraw :: MouseMove( const Point& cPosition, int nCode, uint32 nButtons, Message* pcData )
{
	if( nCode == MOUSE_ENTERED )
	{
		if( SLogicApp::GetAppWindow()->GetCurrentTool() != NULL )
				SLogicApp::GetAppWindow()->GetCurrentTool()->SetCursor();
	}
	else if( nCode == MOUSE_EXITED )
	{
		Application::GetInstance()->PopCursor();
	}
	else
	{
		if( SLogicApp::GetAppWindow()->GetCurrentTool() != NULL )
			SLogicApp::GetAppWindow()->GetCurrentTool()->MouseMove( this, cPosition, nButtons );
	}
}

void CompDraw :: MouseUp( const Point& cPosition, uint32 nButtons, Message* pcData )
{
	if( pcData != NULL)
	{
		Component* comp;

		if( pcData->FindPointer( "component", (void **) &comp ) != 0 )
			return;

		// Find delta
		Point delta;
		if( pcData->FindPoint( "delta", &delta ) != 0 )
			return;
                
		// Create new component       
		Component* m = comp->Duplicate(); 
		m->SetPosition( Snap( cPosition - delta ) );
		AddComponent( m );

		Invalidate( Rect( m->GetPosition() - Point( 3, 3 ), m->GetPosition() + m->GetSize() + Point( 3, 3 )  ) );
		Sync();		
	}
	else
	{
		if( SLogicApp::GetAppWindow()->GetCurrentTool() != NULL )
			SLogicApp::GetAppWindow()->GetCurrentTool()->MouseUp( this, cPosition, nButtons );	
	}
}

void CompDraw :: DrawDigitalLine( Point start, Point end )
{
	DrawLine( start, Point( start.x - ( start.x - end.x ) / 2, start.y ) );
	DrawLine( end, Point( end.x + ( start.x - end.x ) / 2, end.y ) );
	DrawLine( Point( start.x - ( start.x - end.x ) / 2, start.y ),
  			  Point( end.x + ( start.x - end.x ) /2, end.y ) );
}

Point CompDraw :: Snap( Point pos )
{
	if( m_SnapMode )
	{
		pos.x= rint( ( pos.x / m_SnapX ) + 0.5f ) * m_SnapX;
		pos.y= rint( ( pos.y / m_SnapY ) + 0.5f ) * m_SnapY;
	}

	return pos;
}

void CompDraw :: RedrawComponent( Component* comp )
{
	Invalidate( Rect( comp->GetPosition() - Point( 3, 3 ), 
  	  	              comp->GetPosition() + comp->GetSize() + Point( 3, 3 ) ) );
	Sync();
}

void CompDraw :: UpdateSimulation()
{
	m_SimulationTime++;

	for( uint i = 0; i < m_ListOfComponents.size(); i++ )
	{
		if( m_ListOfComponents[ i ]->WaitingForUpdate() )
			m_ListOfComponents[ i ]->UpdateSimulation( m_SimulationTime );
	}
}
