	// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "component.h"
#include "common.h"
#include "pin_io.h"
#include "main.h"

using namespace os;
using namespace std;

Component :: Component( Image* image, Image* dragimage, String type, String description )
{
	m_Name = "";
	m_Type = type;
	m_Description = description;
	m_DragMode = false;
	m_Image = image;
	m_DragImage = dragimage;
	m_DragImage->ApplyFilter( Image::F_GRAY );
	m_Delay = 0;
	m_InputChanged = 0;
	m_Selected = false;
}

Component :: Component( String type, String description )
{
	m_Name = "";
	m_Type = type;
	m_Description = description;
}

Component :: ~Component()
{
}

void Component :: DrawSelectionBox( View* view )
{
	assert( view != NULL );
	assert( m_Image != NULL );

	if( m_Selected )
	{
		view->SetDrawingMode( DM_COPY );
		view->SetFgColor( get_default_color(COL_ICON_SELECTED) );

		Rect cFrame( m_Position, m_Position + GetSize() );

		view->FillRect( cFrame );
                        
		// Round edges
		view->DrawLine( Point( cFrame.left + 2, cFrame.top - 2 ), 
						Point( cFrame.right - 2, cFrame.top - 2 ) );
		view->DrawLine( Point( cFrame.left, cFrame.top - 1 ), 
						Point( cFrame.right, cFrame.top - 1 ) );
                        
		view->DrawLine( Point( cFrame.left - 2, cFrame.top + 2 ), 
						Point( cFrame.left - 2, cFrame.bottom - 2 ) );
		view->DrawLine( Point( cFrame.left - 1, cFrame.top ), 
						Point( cFrame.left - 1, cFrame.bottom ) );
                                                                
		view->DrawLine( Point( cFrame.left + 2, cFrame.bottom + 2 ), 
						Point( cFrame.right - 2, cFrame.bottom + 2 ) );
		view->DrawLine( Point( cFrame.left, cFrame.bottom + 1 ), 
						Point( cFrame.right, cFrame.bottom + 1 ) );
                                                                
		view->DrawLine( Point( cFrame.right + 2, cFrame.top + 2 ), 
						Point( cFrame.right + 2, cFrame.bottom - 2 ) );
		view->DrawLine( Point( cFrame.right + 1, cFrame.top ), 
						Point( cFrame.right + 1, cFrame.bottom ) );
	}
}

void Component :: DrawComponent( View* view )
{
	assert( view != NULL );
	assert( m_Image != NULL );

	view->SetDrawingMode( DM_BLEND );
	if( m_DragMode && m_Selected )
		m_DragImage->Draw( m_Position, view );
	else	
		m_Image->Draw( m_Position, view );
}

void Component :: DrawDragComponent( View* view, Point new_position )
{
	assert( view != NULL );
	assert( m_Image != NULL );

	view->SetDrawingMode( DM_BLEND );
	m_Image->Draw( new_position, view );
}

void Component :: Redraw()
{
	SLogicApp::GetAppWindow()->RedrawComponent( this );
}

Component* Component :: Duplicate()
{
	Component* comp = new Component( m_Type, m_Description );
	return comp;
}

void Component :: GetImages( os::Image*& image, os::Image*& dragimage )
{
	image = m_Image;
	dragimage = m_DragImage;
}

void Component :: SetImages( os::Image* image, os::Image* dragimage )
{
	m_Image = image;
	m_DragImage = dragimage;
}


void Component :: AddInputPin( Point pos, String name )
{
	PinIO* pin = new PinIO( this, PinIO::INPUT, pos, name );
	m_ListOfPins.push_back( pin );
}

void Component :: AddOutputPin( Point pos, String name )
{
	PinIO* pin = new PinIO( this, PinIO::OUTPUT, pos, name );
	m_ListOfPins.push_back( pin );
}

PinIO* Component :: HitCheckPin( Point& where, bool allow_input, bool allow_output )
{
	for( int i = 0 ; i < m_ListOfPins.size() ; i++ )
	{
		PinIO* pin = m_ListOfPins[ i ];

		if( ( pin->IsOutput() && allow_output ) || ( pin->IsInput() && allow_input ) )
		{
			if( pin->HitCheck( where ) )
				return pin;
		}	
	}

	return NULL;
}

bool Component :: GetPinState( String name )
{
	for( int i = 0 ; i < m_ListOfPins.size() ; i++ )
	{
		if( m_ListOfPins[ i ]->GetName() == name )
		{
			return m_ListOfPins[ i ]->GetCurrentState();
		}
	}
}

void Component :: SetPinState( String name, bool state )
{
	for( int i = 0 ; i < m_ListOfPins.size() ; i++ )
	{
		if( m_ListOfPins[ i ]->GetName() == name )
		{
			m_ListOfPins[ i ]->SetNewState( state );
			return;
		}
	}
}

void Component :: UpdateSimulation( int simtime )
{
printf( "Type: %s\n", m_Type.c_str() );

	bool changed = false;

	// Update all inputs
	for( int i = 0 ; i < m_ListOfPins.size() ; i++ )
	{
		if( m_ListOfPins[ i ]->IsInput() && m_ListOfPins[ i ]->NeedsUpdate() )
		{
			if( m_ListOfPins[ i ]->UpdateInput() )
				changed = true;
		}
	}

	// If an input has changed we need to update all outputs
	if( changed )
	{
		for( int i = 0 ; i < m_ListOfPins.size() ; i++ )
		{
			if( m_ListOfPins[ i ]->IsInput() )
			{
				ChangeInputState( m_ListOfPins[ i ]->GetName(), m_ListOfPins[ i ]->GetCurrentState() );
			}
		}	
	}
}
