// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __COMPONENT_H_
#define __COMPONENT_H_

#include <vector>

#include <gui/image.h>
#include <gui/view.h>
#include <util/string.h>

class PinIO;

class Component
{
public:
	Component( os::Image* image, os::Image* dragimage, os::String type, os::String description );
	Component( os::String type, os::String description );
	virtual ~Component();

	os::String GetName() { return m_Name; };
	os::String GetType() { return m_Type; };
	os::String GetDescription() { return m_Description; };
	os::Image* GetImage() { return m_Image; };
	

	void SetPosition( os::Point pos ) { m_Position = pos; };
	os::Point GetPosition() { return m_Position; };
	os::Point GetSize() { return m_Image->GetSize(); };

	void GetImages( os::Image*& image, os::Image*& dragimage );
	void SetImages( os::Image* image, os::Image* dragimage );

	void DrawSelectionBox( os::View* view );
	void DrawComponent( os::View* view );
	void DrawDragComponent( os::View* view, os::Point new_position );
	void Redraw();

	bool IsSelected() { return m_Selected; };
	void SetSelected( bool sel = false ) { m_Selected = sel; };

	void DragMode( bool drag ) { m_DragMode = drag; };

	void AddInputPin( os::Point pos, os::String name = "" );
	void AddOutputPin( os::Point pos, os::String name = "" );

	PinIO* HitCheckPin( os::Point& where, bool allow_input = true, bool allow_output = true );

	bool GetPinState( os::String name );
	void SetPinState( os::String name, bool state );

	virtual void ChangeInputState( os::String pin_name, bool state ) { };
	virtual bool GetOutputState( os::String name ) { return false; };

	virtual Component* Duplicate();

	virtual void Invoked() {};

	void SetDelay( int delay ) { m_Delay = delay; };
	int GetDelay() { return m_Delay; };

	void InputChanged() { m_InputChanged++; };
	void ClearInputChanged() { m_InputChanged--; };
	bool WaitingForUpdate() { return m_InputChanged>0; };

	void UpdateSimulation( int simtime );

private:
	os::String m_Name;
	os::String m_Type;
	os::String m_Description;
	os::String m_Resource;
	bool m_DragMode;
	os::Image* m_Image;
	os::Image* m_DragImage;

	int m_Delay;
	int m_InputChanged;
	bool m_Selected;

	os::Point m_Position;

	std::vector< PinIO* > m_ListOfPins;
};

#endif
