// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "compview.h"
#include "component_and.h"
#include "component_or.h"
#include "component_led.h"
#include "component_vcc.h"
#include "component_gnd.h"
#include "component_button.h"
#include "component_xor.h"
#include "component_not.h"
#include "common.h"

using namespace os;

CompView :: CompView( const Rect& cFrame, uint32 nResizeMask ) : IconView( cFrame, "", nResizeMask )
{
	SetView( VIEW_DETAILS );
	SetMultiSelect( false );

	AddComponent( new ComponentAnd() );
	AddComponent( new ComponentOr() );
	AddComponent( new ComponentLED() );
	AddComponent( new ComponentVCC() );
	AddComponent( new ComponentGND() );
	AddComponent( new ComponentButton() );
	AddComponent( new ComponentXOR() );
	AddComponent( new ComponentNOT() );

	Layout();
}	

CompView :: ~CompView()
{
}

void CompView :: AddComponent( Component* comp )
{
	CompIconData* data = new CompIconData( comp );
	int count = AddIcon( comp->GetImage(), data );
	AddIconString( count, comp->GetType() );
	AddIconString( count, comp->GetDescription() );
}

void CompView :: DragSelection( Point cStartPoint )
{
	Message cData( 1234 );
	int nNumObjects = 0;
	int nLastSelected = 0;
	Point cIconPos = cStartPoint;
        
	/* Collect selected icons */
	for( uint i = 0; i < GetIconCount(); i++ )
	{
		if( GetIconSelected( i ) )
		{                        
			CompIconData *pcData = static_cast < CompIconData * >( GetIconData( i ) );
			cData.AddPointer( "component", pcData->GetComponent() );
			nNumObjects++;
                        
			if( os::Rect( GetIconPosition( i ), GetIconPosition( i ) + GetIconSize() ).DoIntersect( cStartPoint ) )
	 			cIconPos = GetIconPosition( i );
                        
			nLastSelected = i;

		}
	}
        
	cData.AddPoint( "delta", cStartPoint - cIconPos );
        
	// Create drag object
	os::Bitmap cImage( (int)GetIconSize().x + 1, (int)GetIconSize().y + 1, os::CS_RGB32, os::Bitmap::ACCEPT_VIEWS | os::Bitmap::SHARE_FRAMEBUFFER );
	View *pcView = new View( os::Rect( os::Point( 0, 0 ), GetIconSize() ), "" );
	cImage.AddChild( pcView );        
	os::Image* pcIcon = NULL;               
	String icon_string;

	if( nNumObjects == 1 && GetIconImage( nLastSelected ) )
	{     
		pcIcon =  GetIconImage( nLastSelected );
		icon_string = GetIconString( nLastSelected, 0 );
	}
	else
	{
		pcIcon = LoadImage( "close_button.png" );
		icon_string.Format( "%i components", nNumObjects );
	}

	if( pcIcon )
		RenderIcon( icon_string, pcIcon, pcView, os::Point( 0, 0 ) );

	cImage.Sync();

	if( nNumObjects != 1 )
		delete( pcIcon );

	BeginDrag( &cData, cStartPoint - cIconPos, &cImage );
}

