// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "pin_io.h"

using namespace os;

PinIO :: PinIO( Component* parent, enum IOType type, Point position, String name )
{
	m_Parent = parent;
	m_IOType = type;
	m_Position = position;
	m_Name = name;
	m_ConnectedTo = NULL;

	m_State = false;
}

bool PinIO :: HitCheck( Point check_point )
{
	Rect t( m_Parent->GetPosition() + m_Position - Point( 5, 5 ), m_Parent->GetPosition() + m_Position + Point( 5, 5 ) );
	return ( t.DoIntersect( check_point ) );
}

void PinIO :: ConnectTo( PinIO* pin )
{
	m_ConnectedTo = pin;

	if( pin->IsOutput() && IsInput() )
		SetNewState( pin->GetCurrentState() );
}

bool PinIO :: GetCurrentState()
{
	if( m_IOType == PinIO::INPUT )
		return m_State;
	else
		return m_Parent->GetOutputState( m_Name );
}

bool PinIO :: SetNewState( bool n )
{
	if( n != m_State )
	{		
		if( m_IOType == PinIO::INPUT )
		{
	printf( "SetNewState %s\n", m_Parent->GetType().c_str() );

			m_NewState = n;
			SetPinDelay( m_Parent->GetDelay() );
			m_Parent->InputChanged();
		}
	
		if( m_IOType == PinIO::OUTPUT )
		{
			m_State = n;
			UpdateOutput();
		}
	}
}

bool PinIO :: UpdateInput()
{
printf( "Pin::Updateinput\n" );

	if( DecreasePinDelay() )
	{
printf( "Pin::UpdateInput cinput changed!\n" );
		m_State = m_NewState;
		m_Parent->ClearInputChanged();
		return true;
	}

	return false;
}

void PinIO :: UpdateOutput()
{
printf( "Updating output\n" );
	if( m_ConnectedTo != NULL )
		m_ConnectedTo->SetNewState( m_State );
	
}
