// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef __PIN_IO_H_
#define __PIN_IO_H_

#include <gui/point.h>
#include <util/string.h>
#include "component.h"

class Component;

class PinIO
{
public:
	enum IOType { INPUT, OUTPUT };

	PinIO( Component* parent, enum IOType type, os::Point position, os::String name = "" );
	~PinIO() {};

	os::String GetName() { return m_Name; };

	bool IsOutput() { return (m_IOType == OUTPUT); };
	bool IsInput() { return (m_IOType == INPUT); };

	bool HitCheck( os::Point check_point );
	os::Point GetPosition() { return ( m_Parent->GetPosition() + m_Position ); };

	void ConnectTo( PinIO* pin );

	bool GetCurrentState();
	bool SetNewState( bool n );
	bool NeedsUpdate() { return m_State != m_NewState; };
	bool ApplyNewState() { m_State = m_NewState; };
	bool UpdateInput();
	void UpdateOutput();

	void SetState( bool state );

	void SetPinDelay( int delay ) { m_PinDelay = delay; };
	bool DecreasePinDelay() { if( m_PinDelay > 0 ) m_PinDelay--; return ( m_PinDelay == 0 ); };

private:
	Component* m_Parent;
	enum IOType m_IOType;
	os::Point m_Position;
	os::String m_Name;

	bool m_State;
	bool m_NewState;

	int m_PinDelay;

	PinIO* m_ConnectedTo;
};

#endif
