// SLogic	 (C)opyright 2006 Jonas Jarvoll
//
// This is free software; you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "tool_arrow.h"
#include "component.h"
#include "compdraw.h"

using namespace os;

ToolArrow :: ToolArrow() : Tool( 0 )
{
}

ToolArrow :: ~ToolArrow()
{
}

void ToolArrow :: Init()
{
	m_bMouseDown = false;
	m_bMouseDownOverIcon = false;
	m_bSelecting = false;
	m_bDragging = false;
	m_nLastClick = 0;
}

void ToolArrow :: MouseUp( CompDraw* draw, const Point& cPosition, uint32 nButtons )
{
	m_bMouseDown = false;
	m_bMouseDownOverIcon = false;

	if( m_bSelecting )
	{
		// Select this icons
		draw->Select( m_cSelectStart, m_cLastSelectPosition, draw->GetQualifiers() & os::QUAL_CTRL );
//                        m->m_pcControl->SelectionChanged();

		// Swap positions if necessary
		if( m_cSelectStart.x > m_cLastSelectPosition.x )
		{
			float vTemp = m_cSelectStart.x;
			m_cSelectStart.x = m_cLastSelectPosition.x;
			m_cLastSelectPosition.x = vTemp;
		}       
		if( m_cSelectStart.y > m_cLastSelectPosition.y ) 
		{
			float vTemp = m_cSelectStart.y;
			m_cSelectStart.y = m_cLastSelectPosition.y;
			m_cLastSelectPosition.y = vTemp;
		}          

		draw->Invalidate( Rect( m_cSelectStart - Point( 3, 3 ), m_cLastSelectPosition + Point( 3, 3 ) ) );
		draw->Flush();                
	}
            
	if( m_bDragging )
	{		
		// Fix new position of the components
		for( uint i = 0; i < draw->m_ListOfComponents.size(); i++ )
		{
			if( draw->m_ListOfComponents[ i ]->IsSelected() )
			{
				// Invalidate old position to force a redraw
				draw->Invalidate( Rect( draw->m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ), 
				       		  	  draw->m_ListOfComponents[ i ]->GetPosition() + draw->m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) ) );		
				draw->m_ListOfComponents[ i ]->DragMode( false );
				draw->m_ListOfComponents[ i ]->SetPosition( draw->Snap( draw->m_ListOfComponents[ i ]->GetPosition() - m_cOldRelativePosition ) );

				// Invalidate new position to force a redraw
				draw->Invalidate( Rect( draw->m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ), 
				       		  	        draw->m_ListOfComponents[ i ]->GetPosition() + draw->m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) ) );		
			}
		}

		draw->Sync();	
	} 

	m_bDragging = false;
	m_bSelecting = false;
}

void ToolArrow :: MouseDown( CompDraw* draw, const Point& cPosition, uint32 nButtons )
{
	int nComp = 0;

	draw->MakeFocus();

	m_bMouseDown = true;
	m_bSelecting = false;
                        
	if( ( nComp = draw->HitTest( cPosition ) ) < 0 )
	{
		// Deselect all icons if ctrl is not pressed
		if( !( draw->GetQualifiers() & os::QUAL_CTRL ) )
		{
			draw->DeselectAll();
//                                m->m_pcControl->SelectionChanged();
			draw->Flush();
		}
		
		m_bMouseDownOverIcon = false;
		m_cSelectStart = cPosition;
                        
		if( nButtons == 1 ) 
			return;
	}

	// Open context menu...
	if( nButtons != 1 )
	{
		//std::cout<<"Open context menu!!"<<std::endl;
		if( nComp >= 0 )
		{
			if( draw->GetQualifiers() & os::QUAL_CTRL )
			{
				// Select/Deselect this icon
				draw->Select( nComp, !draw->m_ListOfComponents[ nComp ]->IsSelected() );
			}
			else 
			{
				if( !draw->m_ListOfComponents[ nComp ]->IsSelected() )
					// Deselect all icons
					draw->DeselectAll();
                                        
				// Select this icon
				draw->Select( nComp, true );
			}
		}
        //                m->m_pcControl->SelectionChanged();
		draw->Flush();
                        
	//	m->m_pcControl->OpenContextMenu( cPosition + m->m_pcView->GetScrollOffset(), nIcon >= 0 );
printf( "Context menu\n" );
		return;
	}

	// Check if we have a double click
	if( get_system_time() - m_nLastClick < 500000 && draw->m_ListOfComponents[ nComp ]->IsSelected() )
	{
		// Deselect all icons
		draw->DeselectAll();

		// Select this icon
		draw->Select( nComp, true );
//                        m->m_pcControl->SelectionChanged();

		draw->Flush();
                        
		m_bMouseDown = false;
		return;
	}

	// Handle single click
	m_nLastClick = get_system_time();
	m_bMouseDownOverIcon = true;

	if( ( draw->GetQualifiers() & os::QUAL_CTRL ) ) 
	{
		// Select/Deselect this icon
		draw->Select( nComp, !draw->m_ListOfComponents[ nComp ]->IsSelected() );
	}
	else
	{
		if( !draw->m_ListOfComponents[ nComp ]->IsSelected() )
			// Deselect all icons
			draw->DeselectAll();

		// Select this icon */
		draw->Select( nComp, true );
	}
//                m->m_pcControl->SelectionChanged();
	draw->Flush();
}

void ToolArrow :: MouseMove( CompDraw* draw, const Point& cPosition, uint32 nButtons )
{
	if( !m_bMouseDown || !( nButtons & 1 ) )     
		return;
                        
	// Check what we should do
	if( !m_bMouseDownOverIcon )
	{
		if( !m_bSelecting )
			m_bSelecting = true;
		else
		{
			// Delete old rect
			draw->SetDrawingMode( os::DM_INVERT );
			draw->DrawFrame( Rect( m_cSelectStart, m_cLastSelectPosition ), os::FRAME_THIN | os::FRAME_TRANSPARENT );
		}
                        
		// Select the icons
		draw->Select( m_cSelectStart, cPosition, draw->GetQualifiers() & os::QUAL_CTRL );

		// Draw new frame
		draw->SetDrawingMode( os::DM_INVERT );
		draw->DrawFrame( Rect( m_cSelectStart, cPosition ), os::FRAME_THIN | os::FRAME_TRANSPARENT );

		m_cLastSelectPosition = cPosition;

		draw->Flush();
	} 
	else 
	{
		// Start drag operation
		if( !m_bDragging )
		{
			for( uint i = 0; i < draw->m_ListOfComponents.size(); i++ )
			{
				if( draw->m_ListOfComponents[ i ]->IsSelected() )
				{
					draw->m_ListOfComponents[ i ]->DragMode( true );			
	
					draw->Invalidate( Rect( draw->m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ), 
			        	 	  	    		draw->m_ListOfComponents[ i ]->GetPosition() + draw->m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) ) );
				}
			}

			m_cStartDragPosition = cPosition;
            m_bDragging = true;
		}
		else
		{
			// Calculate relative drag position
			Point rel = m_cStartDragPosition - cPosition;

			for( uint i = 0; i < draw->m_ListOfComponents.size(); i++ )
			{			
				if( draw->m_ListOfComponents[ i ]->IsSelected() )
				{
				//	Invalidate( Rect( draw->m_ListOfComponents[ i ]->GetPosition() - Point( 3, 3 ) - m_cOldRelativePosition, 
				  //      		  	  draw->m_ListOfComponents[ i ]->GetPosition() + draw->m_ListOfComponents[ i ]->GetSize() + Point( 3, 3 ) - m_cOldRelativePosition ) );
				}
			}

			for( uint i = 0; i < draw->m_ListOfComponents.size(); i++ )
			{			
				if( draw->m_ListOfComponents[ i ]->IsSelected() )
				{
					Point pos = draw->m_ListOfComponents[ i ]->GetPosition() - rel;
					
					draw->m_ListOfComponents[ i ]->DrawDragComponent( draw, draw->Snap( pos ) );
				}
			}

			m_cOldRelativePosition = rel;

			draw->Sync();
		}
	}
}
